#!/usr/bin/env python

""" Convert absolute local paths in the #! line, to relative paths """

import os
import sys

def convert_file(file):
    if os.path.islink(file):
        return

    input = open(file, 'r')
    shebang = input.readline()
    if not shebang.startswith('#!'):
        return
    shebang = shebang[2:].strip()
   
    args = shebang.split(' ')
    absolute = args[0]
    dir = os.path.abspath(os.path.dirname(file))
    if not absolute.startswith(dir):
        return
    relative = absolute.replace(dir, '.').strip()

    params = " ".join(args[1:])
    if params:
        params = " " + params

    stat = os.stat(file)
    mode = stat[0]
    times = stat[7:9]
    real = ".%s" % os.path.basename(file)
    path = os.path.join(dir, real)
   
    SH = """#!/bin/sh
progname="$0"
curdir=`dirname "$progname"`
exec -a "$progname" "$curdir/%s"%s "$curdir/%s" "$@"\
""" % (relative, params, real)

    print "Converting %s" % file
    
    output = open(path, 'w')
    print >>output, "#!%s%s" % (relative, params)
    for line in input:
        output.write(line)
    output.close()
    os.utime(path, times)
 
    script = open(file, 'w')
    script.write(SH)
    script.close()
    os.chmod(file, mode)

for path in sys.argv[1:]:
    if os.path.isfile(path):
        convert_file(path)
    elif os.path.isdir(path):
        for root, dirs, files in os.walk(path):
            for name in files:
                convert_file(os.path.join(root, name))

